@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* test.c: test builtin */
-/* (C) 2007-2018 magicant */
+/* (C) 2007-2020 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,16 +77,28 @@ static enum filecmp compare_files(const wchar_t *left, const wchar_t *right)
 #if YASH_ENABLE_DOUBLE_BRACKET
 static int eval_dbexp(const dbexp_T *e)
     __attribute__((nonnull));
-static inline wchar_t *expand_double_bracket_operand(const wordunit_T *w)
+static inline cc_word_T expand_double_bracket_operand(const wordunit_T *w)
+    __attribute__((nonnull,warn_unused_result));
+static inline wchar_t *expand_double_bracket_operand_unescaped(
+	const wordunit_T *w)
     __attribute__((nonnull,malloc,warn_unused_result));
-static wchar_t *expand_and_unescape_double_bracket_operand(const wordunit_T *w)
+static bool test_triple_db(const wchar_t *lhs, const wchar_t *op,
+	const wchar_t *rhsvalue, const char *rhscc)
+    __attribute__((nonnull));
+static bool quote_removal_and_test_triple(
+	const wchar_t *lhs, const wchar_t *op,
+	const wchar_t *rhsvalue, const char *rhscc)
+    __attribute__((nonnull));
+static bool quote_removal_and_pattern_matching(
+	const wchar_t *lhs, const wchar_t *rhsvalue, const char *rhscc)
+    __attribute__((nonnull));
+static bool quote_removal_and_regex_matching(
+	const wchar_t *lhs, const wchar_t *rhsvalue, const char *rhscc)
+    __attribute__((nonnull));
+static wchar_t *quote_removal_for_regex(const wchar_t *s, const char *cc)
     __attribute__((nonnull,malloc,warn_unused_result));
-static bool test_triple_db(
-	const wchar_t *lhs, const wchar_t *op, const wchar_t *rhs_escaped)
-    __attribute__((nonnull));
-static bool test_triple_args(
-	const wchar_t *left, const wchar_t *op, const wchar_t *right)
-    __attribute__((nonnull));
+static const wchar_t *skip_bracket(const wchar_t *s)
+    __attribute__((nonnull,pure,warn_unused_result));
 #endif
 
 
@@ -723,7 +735,8 @@ int eval_dbexp(const dbexp_T *e)
 {
     int lhs_result;
     bool result;
-    wchar_t *lhs = NULL, *rhs = NULL;
+    wchar_t *lhs = NULL;
+    cc_word_T rhs = { NULL, NULL };
 
     switch (e->type) {
 	case DBE_OR:
@@ -744,25 +757,27 @@ int eval_dbexp(const dbexp_T *e)
 	    }
 
 	case DBE_UNARY:
-	    rhs = expand_and_unescape_double_bracket_operand(e->rhs.word);
-	    if (rhs == NULL)
+	    rhs.value = expand_double_bracket_operand_unescaped(e->rhs.word);
+	    if (rhs.value == NULL)
 		return Exit_TESTERROR;
-	    result = test_double((void *[]) { e->operator, rhs });
+	    result = test_double((void *[]) { e->operator, rhs.value });
 	    break;
 	case DBE_BINARY:
-	    lhs = expand_and_unescape_double_bracket_operand(e->lhs.word);
+	    lhs = expand_double_bracket_operand_unescaped(e->lhs.word);
 	    if (lhs == NULL)
 		return Exit_TESTERROR;
 	    rhs = expand_double_bracket_operand(e->rhs.word);
-	    if (rhs == NULL)
+	    if (rhs.value == NULL) {
+		free(lhs);
 		return Exit_TESTERROR;
-	    result = test_triple_db(lhs, e->operator, rhs);
+	    }
+	    result = test_triple_db(lhs, e->operator, rhs.value, rhs.cc);
 	    break;
 	case DBE_STRING:
-	    rhs = expand_and_unescape_double_bracket_operand(e->rhs.word);
-	    if (rhs == NULL)
+	    rhs.value = expand_double_bracket_operand_unescaped(e->rhs.word);
+	    if (rhs.value == NULL)
 		return Exit_TESTERROR;
-	    result = test_single((void *[]) { rhs });
+	    result = test_single((void *[]) { rhs.value });
 	    break;
 
 	default:
@@ -770,34 +785,31 @@ int eval_dbexp(const dbexp_T *e)
     }
 
     free(lhs);
-    free(rhs);
+    free(rhs.value);
+    free(rhs.cc);
     if (yash_error_message_count > 0)
 	return Exit_TESTERROR;
     return result ? Exit_TRUE : Exit_FALSE;
 }
 
-/* Expands the operand of a primary.
- * The result may contain backslash escapes. */
-wchar_t *expand_double_bracket_operand(const wordunit_T *w)
+/* Expands the operand of a primary, but without quote removal. */
+cc_word_T expand_double_bracket_operand(const wordunit_T *w)
 {
-    return expand_single(w, TT_SINGLE, true, false);
+    return expand_single_cc(w, TT_SINGLE, Q_WORD);
 }
 
 /* Expands the operand of a primary.
  * The result is literal (does not contain backslash escapes). */
-wchar_t *expand_and_unescape_double_bracket_operand(const wordunit_T *w)
+wchar_t *expand_double_bracket_operand_unescaped(const wordunit_T *w)
 {
-    wchar_t *e = expand_double_bracket_operand(w);
-    if (e == NULL)
-	return NULL;
-    return unescapefree(e);
+    return expand_single(w, TT_SINGLE, Q_WORD, ES_NONE);
 }
 
 /* Tests the specified three-token (binary) primary in the double-bracket
- * command. The left-hand-side must be given literal while the right-hand-side
- * backslash-escaped. */
-bool test_triple_db(
-	const wchar_t *lhs, const wchar_t *op, const wchar_t *rhs_escaped)
+ * command. The left-hand-side must be given literal (with quote removal already
+ * performed) while the right-hand-side quoted (without quote removal). */
+bool test_triple_db(const wchar_t *lhs, const wchar_t *op,
+	const wchar_t *rhsvalue, const char *rhscc)
 {
     /* Some string comparison primaries in the double-bracket command are
      * different from those in the test built-in. */
@@ -805,30 +817,132 @@ bool test_triple_db(
 	case L'=':
 	    if (op[1] == L'~') {
 		assert(op[2] == L'\0');
-		return test_triple_args(lhs, op, rhs_escaped);
+		return quote_removal_and_regex_matching(lhs, rhsvalue, rhscc);
 	    }
 	    if (op[1] == L'\0' || (op[1] == L'=' && op[2] == L'\0'))
-		return match_pattern(lhs, rhs_escaped);
+		return quote_removal_and_pattern_matching(lhs, rhsvalue, rhscc);
 	    break;
 	case L'!':
 	    assert(op[1] == L'=');
 	    if (op[2] == L'\0')
-		return !match_pattern(lhs, rhs_escaped);
+		return
+		    !quote_removal_and_pattern_matching(lhs, rhsvalue, rhscc);
 	    break;
     }
 
-    wchar_t *rhs = unescape(rhs_escaped);
-    bool result = test_triple_args(lhs, op, rhs);
+    return quote_removal_and_test_triple(lhs, op, rhsvalue, rhscc);
+}
+
+/* Performs quote removal on the right hand side and then applies `test_triple'.
+ */
+bool quote_removal_and_test_triple(
+	const wchar_t *lhs, const wchar_t *op,
+	const wchar_t *rhsvalue, const char *rhscc)
+{
+    wchar_t *rhs = quote_removal(rhsvalue, rhscc, ES_NONE);
+    void *args[] = { (void *) lhs, (void *) op, (void *) rhs, };
+    bool result = test_triple(args);
     free(rhs);
     return result;
 }
 
-/* Tests the specified three-token expression. */
-bool test_triple_args(
-	const wchar_t *left, const wchar_t *op, const wchar_t *right)
+/* Performs quote removal on the right hand side and then pattern matching. */
+bool quote_removal_and_pattern_matching(
+	const wchar_t *lhs, const wchar_t *rhsvalue, const char *rhscc)
 {
-    void *args[] = { (void *) left, (void *) op, (void *) right, };
-    return test_triple(args);
+    wchar_t *rhs = quote_removal(rhsvalue, rhscc, ES_QUOTED);
+    bool result = match_pattern(lhs, rhs);
+    free(rhs);
+    return result;
+}
+
+/* Performs quote removal on the right hand side and then regular expression
+ * matching. */
+bool quote_removal_and_regex_matching(
+	const wchar_t *lhs, const wchar_t *rhsvalue, const char *rhscc)
+{
+    wchar_t *rhs = quote_removal_for_regex(rhsvalue, rhscc);
+    bool result = match_regex(lhs, rhs);
+    free(rhs);
+    return result;
+}
+
+/* Removes all quotation marks in the input string `s' and add backslash escapes
+ * to quoted characters that would otherwise be treated specially when parsed as
+ * an extended regular expression pattern. The result is a newly malloced
+ * string. */
+wchar_t *quote_removal_for_regex(const wchar_t *s, const char *cc)
+{
+    /* First, remove quotations. */
+    xwcsbuf_T tmp;
+    xstrbuf_T tmpcc;
+    size_t sizehint = wcslen(s);
+    wb_initwithmax(&tmp, sizehint);
+    sb_initwithmax(&tmpcc, sizehint);
+    for (size_t i = 0; s[i] != L'\0'; i++) {
+	if (cc[i] & CC_QUOTATION)
+	    continue;
+	wb_wccat(&tmp, s[i]);
+	sb_ccat(&tmpcc, cc[i]);
+    }
+
+    /* Next, escape unquoted special chars outside brackets */
+    xwcsbuf_T result;
+    wb_initwithmax(&result, sizehint);
+    for (size_t i = 0; tmp.contents[i] != L'\0'; ) {
+	if (tmpcc.contents[i] & CC_QUOTED) {
+	    if (wcschr(L"^.[$()|*+?{\\", tmp.contents[i]) != NULL)
+		wb_wccat(&result, L'\\');
+	    wb_wccat(&result, tmp.contents[i++]);
+	} else {
+	    if (tmp.contents[i] != L'[') {
+		wb_wccat(&result, tmp.contents[i++]);
+	    } else {
+		const wchar_t *s2 = skip_bracket(&tmp.contents[i]);
+		size_t j = s2 - tmp.contents;
+		while (i < j)
+		    wb_wccat(&result, tmp.contents[i++]);
+	    }
+	}
+    }
+
+    sb_destroy(&tmpcc);
+    wb_destroy(&tmp);
+    return wb_towcs(&result);
+}
+
+/* Skips a bracket expression in a regular expression pattern.
+ * Returns a pointer to the character just after the closing L']' (or the
+ * terminating null character). */
+const wchar_t *skip_bracket(const wchar_t *s)
+{
+    assert(*s == L'[');
+    s++;
+
+    if (*s == L'^')
+	s++;
+    if (*s == L']')
+	s++;
+
+    while (*s != L'\0') {
+	if (*s == L']')
+	    return s + 1;
+	if (*s++ != L'[')
+	    continue;
+
+	switch (*s) {
+	    case L':': case L'.': case L'=': ;
+		wchar_t end[] = { *s, L']', L'\0', };
+		s++;
+		const wchar_t *endp = wcsstr(s, end);
+		if (endp == NULL)
+		    return s + wcslen(s);
+		s = endp + 2;
+		break;
+	}
+    }
+
+    return s;
 }
 
 #endif /* YASH_ENABLE_DOUBLE_BRACKET */
