@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* complete.c: command line completion */
-/* (C) 2007-2016 magicant */
+/* (C) 2007-2022 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1374,7 +1374,9 @@ const struct xgetopt_T complete_options[] = {
     { L'D', L"description",          OPTARG_REQUIRED, true,  NULL, },
     { L'd', L"directory",            OPTARG_NONE,     true,  NULL, },
     { L'-', L"dirstack-index",       OPTARG_NONE,     true,  NULL, },
+    { L'-', L"elective-builtin",     OPTARG_NONE,     true,  NULL, },
     { L'-', L"executable-file",      OPTARG_NONE,     true,  NULL, },
+    { L'-', L"extension-builtin",    OPTARG_NONE,     true,  NULL, },
     { L'-', L"external-command",     OPTARG_NONE,     true,  NULL, },
     { L'f', L"file",                 OPTARG_NONE,     true,  NULL, },
     { L'-', L"finished-job",         OPTARG_NONE,     true,  NULL, },
@@ -1387,6 +1389,7 @@ const struct xgetopt_T complete_options[] = {
     { L'h', L"hostname",             OPTARG_NONE,     true,  NULL, },
     { L'j', L"job",                  OPTARG_NONE,     true,  NULL, },
     { L'k', L"keyword",              OPTARG_NONE,     true,  NULL, },
+    { L'-', L"mandatory-builtin",    OPTARG_NONE,     true,  NULL, },
     { L'T', L"no-termination",       OPTARG_NONE,     true,  NULL, },
     { L'-', L"normal-alias",         OPTARG_NONE,     true,  NULL, },
     { L'O', L"option",               OPTARG_NONE,     true,  NULL, },
@@ -1399,6 +1402,7 @@ const struct xgetopt_T complete_options[] = {
     { L'-', L"signal",               OPTARG_NONE,     true,  NULL, },
     { L'-', L"special-builtin",      OPTARG_NONE,     true,  NULL, },
     { L'-', L"stopped-job",          OPTARG_NONE,     true,  NULL, },
+    { L'-', L"substitutive-builtin", OPTARG_NONE,     true,  NULL, },
     { L'S', L"suffix",               OPTARG_REQUIRED, true,  NULL, },
     { L'u', L"username",             OPTARG_NONE,     true,  NULL, },
     { L'v', L"variable",             OPTARG_NONE,     true,  NULL, },
@@ -1466,10 +1470,23 @@ int complete_builtin(int argc __attribute__((unused)), void **argv)
 		    case L'b':  cgtype |= CGT_BINDKEY;  break;
 		    case L'd':  cgtype |= CGT_DIRSTACK;  break;
 		    case L'e':
-			assert(opt->longopt[1] == L'x');
-			switch (opt->longopt[2]) {
-			    case L'e':  cgtype |= CGT_EXECUTABLE;  break;
-			    case L't':  cgtype |= CGT_EXTCOMMAND;  break;
+			switch (opt->longopt[1]) {
+			    case L'l':  cgtype |= CGT_LBUILTIN;  break;
+			    case L'x':
+				switch (opt->longopt[2]) {
+				case L'e':  cgtype |= CGT_EXECUTABLE;  break;
+				case L't':
+				    assert(opt->longopt[3] == L'e');
+				    switch (opt->longopt[4]) {
+					case L'n':
+					    cgtype |= CGT_XBUILTIN;    break;
+					case L'r':
+					    cgtype |= CGT_EXTCOMMAND;  break;
+				    }
+				    break;
+				default:    assert(false);
+				}
+				break;
 			    default:    assert(false);
 			}
 			break;
@@ -1486,21 +1503,27 @@ int complete_builtin(int argc __attribute__((unused)), void **argv)
 			exitstatus = print_builtin_help(ARGV(0));
 			goto finish;
 #endif
-		    case L'n':  cgtype |= CGT_NALIAS;  break;
+		    case L'm':  cgtype |= CGT_MBUILTIN;  break;
+		    case L'n':  cgtype |= CGT_NALIAS;    break;
 		    case L'r':
 			switch (opt->longopt[1]) {
-			    case L'e':  cgtype |= CGT_RBUILTIN;  break;
+			    case L'e':
+				cgtype |= CGT_XBUILTIN | CGT_UBUILTIN;
+				break;
 			    case L'u':  cgtype |= CGT_RUNNING;   break;
 			    default:    assert(false);
 			}
 			break;
 		    case L's':
 			switch (opt->longopt[1]) {
-			    case L'c':  cgtype |= CGT_SCALAR;     break;
-			    case L'e':  cgtype |= CGT_SSBUILTIN;  break;
-			    case L'i':  cgtype |= CGT_SIGNAL;     break;
-			    case L'p':  cgtype |= CGT_SBUILTIN;   break;
-			    case L't':  cgtype |= CGT_STOPPED;    break;
+			    case L'c':  cgtype |= CGT_SCALAR;    break;
+			    case L'e':
+				cgtype |= CGT_MBUILTIN | CGT_LBUILTIN;
+				break;
+			    case L'i':  cgtype |= CGT_SIGNAL;    break;
+			    case L'p':  cgtype |= CGT_SBUILTIN;  break;
+			    case L't':  cgtype |= CGT_STOPPED;   break;
+			    case L'u':  cgtype |= CGT_UBUILTIN;  break;
 			    default:    assert(false);
 			}
 			break;
